module Grants
  class Notification # fremde nachrichten!

      def self.grants
        {
          read: {
              name: 'Anzeigen',
              scopes: {
                  system_wide: {
                      name: 'Systemweit',
                      user_types: [::Manager],
                      default_for: [],
                      manager: -> ability do
                        ability.can :read, ::Notification
                      end
                  },
                  own_institutions: {
                      name: 'in eigenen Einrichtungen',
                      user_types: [::Manager,::Teacher],
                      default_for:  [::Manager],
                      manager: -> ability, manager do
                        ability.can :read, ::Notification, {
                            institution: {
                                all_parent_institutions: {
                                    parent_id: manager.institution_id
                                }
                            }
                        }
                      end,
                      teacher: -> ability, teacher do
                        ability.can :read, ::Notification, {
                            institution:{
                                teacher_institution_relations: {
                                    teacher_id: teacher.id
                                }
                            }
                        }
                      end
                  },
                  leaded_institutions: {
                      name: 'in administrierten Einrichtungen (Schuladmin)',
                      user_types: [::Teacher],
                      default_for:  [::Teacher],
                      teacher: -> ability, teacher do
                        ability.can :read, ::Notification, {
                            institution:{
                                teacher_institution_relations: {
                                    is_school_admin: true,
                                    teacher_id: teacher.id
                                }
                            }
                        }
                      end
                  }
              }
          },
          destroy: {
              name: 'Löschen',
              scopes: {
                  system_wide: {
                      name: 'Systemweit',
                      user_types: [::Manager],
                      default_for: [],
                      manager: -> ability do
                        ability.can :destroy, ::Notification
                      end
                  },
                  own_institutions: {
                      name: 'in eigenen Einrichtungen',
                      user_types: [::Manager,::Teacher],
                      default_for:  [::Manager],
                      manager: -> ability, manager do
                        ability.can :destroy, ::Notification, {
                            institution: {
                                all_parent_institutions: {
                                    parent_id: manager.institution_id
                                }
                            }
                        }
                      end,
                      teacher: -> ability, teacher do
                        ability.can :destroy, ::Notification, {
                            institution:{
                                teacher_institution_relations: {
                                    teacher_id: teacher.id
                                }
                            }
                        }
                      end
                  },
                  leaded_institutions: {
                      name: 'in administrierten Einrichtungen (Schuladmin)',
                      user_types: [::Teacher],
                      default_for:  [::Teacher],
                      teacher: -> ability, teacher do
                        ability.can :destroy, ::Notification, {
                            institution:{
                                teacher_institution_relations: {
                                    is_school_admin: true,
                                    teacher_id: teacher.id
                                }
                            }
                        }
                      end
                  }
              }
          }
        }
      end
  end
end