class CareerItemsController < ApplicationController

  before_action :set_client, only: [:index, :edit, :update]
  before_action :set_career_item, only: [:edit, :update]

  def index
    # authorize!(:show, CareerItem)
    add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
    add_breadcrumb "#{view_context.show_person(@client)}", dashboard_client_path(@client), title: "Übersicht anzeigen"
    add_breadcrumb "Laufbahn", client_career_items_path(@client), title: "gesamte Laufbahn anzeigen"

    @career_items_grid = CareerItemsGrid.new(params[:career_items_grid]) do |scope|
      scope.where(client_id: params[:client_id]).page(params[:page]).per(4)
    end
  end

  def edit
    # authorize!(:update, @career_item)
    add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
    add_breadcrumb "#{view_context.show_person(@career_item.client)}", dashboard_client_path(@career_item.client), title: "Übersicht anzeigen"
    add_breadcrumb "Laufbahn", client_career_items_path(@career_item.client), title: "gesamte Laufbahn anzeigen"
    add_breadcrumb "Eintrag bearbeiten", edit_client_career_item_path(@client, @career_item), title: "Laufbahn-Eintrag bearbeiten"
  end

  def update
    @career_item.assign_attributes(career_item_params)
    # authorize!(:update, @career_item)
    respond_to do |format|
      if @career_item.save
        format.html { redirect_to client_career_items_path(@career_item.client), success: "Gespeichert" }
      else
        format.html { render :edit }
      end
    end
  end

  private
    # Use callbacks to share common setup or constraints between actions.

    def career_item_params
      params.require(:career_item).permit(:description)
    end

    def set_career_item
      @career_item = CareerItem.find(params[:id])
    end

    def set_client
      @client = Client.find(params[:client_id])
    end
end
