class InstitutionDocumentsController < ApplicationController

  def index
    authorize!(:read, InstitutionDocument)
    @institution= find_institution
    add_breadcrumb @institution.name, institution_path(@institution)
    add_breadcrumb "Dateiablage"
    @grid = InstitutionDocumentsGrid.new(params[:institution_documents_grid]) do |scope|
      scope.where(institution_id: params[:institution_id])
           .where(parent_id: nil)
           .accessible_by(current_ability).page(params[:page]).per(25)
    end
  end

  def show
    @institution_document = find_institution_document
    authorize!(:read,@institution_document)
    @institution = @institution_document.institution
    add_breadcrumb @institution.name, institution_path(@institution)
    add_breadcrumb "Dateiablage", institution_institution_documents_path(institution_id: @institution_document.institution.id)
    @institution_document.all_parent_folders.each do |folder|
      add_breadcrumb folder.description, institution_document_path(folder)
    end
    add_breadcrumb @institution_document.description, institution_document_path(@institution_document)
    @grid = InstitutionDocumentsGrid.new(params[:institution_documents_grid]) do |scope|
      scope.where(institution_id: @institution.id)
           .where(parent_id: @institution_document.id)
           .accessible_by(current_ability).page(params[:page]).per(25)
    end
    render :index
  end

  def edit
    @institution_document = find_institution_document
    authorize!(:update, @institution_document)
    @institution = @institution_document.institution
    add_breadcrumb @institution.name, institution_path(@institution)
    add_breadcrumb "Dateiablage", institution_institution_documents_path(institution_id: @institution_document.institution.id)
    @institution_document.all_parent_folders.each do |folder|
      add_breadcrumb folder.description, institution_document_path(folder)
    end
    add_breadcrumb @institution_document.description, institution_document_path(@institution_document)
    render :edit
  end

  def update
    @institution_document = find_institution_document
    authorize!(:update, @institution_document)
    respond_to do |format|
      if @institution_document.parent_id.present?
        redirect_target = institution_document_path(@institution_document.parent_id)
      else
        redirect_target = institution_institution_documents_path(institution_id: @institution_document.institution.id)
      end
      if @institution_document.update(institution_document_params)
        format.html { redirect_to redirect_target , success: "Dokument wurde erfolgreich gespeichert." }
      else
        errors = @institution_document.errors.full_messages
        format.html { redirect_to redirect_target, danger: "Bitte prüfen Sie Ihre Eingaben. <br/>".html_safe + errors.join('<br/>').html_safe }
      end
    end
  end

  def create
    @institution_document = InstitutionDocument.new(institution_document_params)
    @institution_document.edited_by=current_user.person
    authorize!(:create, @institution_document)
    respond_to do |format|
      if @institution_document.parent_id.present?
        redirect_target = institution_document_path(@institution_document.parent_id)
      else
        redirect_target = institution_institution_documents_path(institution_id: @institution_document.institution.id)
      end
      if @institution_document.save
        format.html { redirect_to redirect_target , success: "Dokument wurde erfolgreich gespeichert." }
      else
        errors = @institution_document.errors.full_messages
        format.html { redirect_to redirect_target, danger: "Bitte prüfen Sie Ihre Eingaben. <br/>".html_safe + errors.join('<br/>').html_safe }
      end
    end
  end

  def destroy
    @institution_document = find_institution_document
    if @institution_document.parent_id.present?
      redirect_target = institution_document_path(@institution_document.parent_id)
    else
      redirect_target = institution_institution_documents_path(institution_id: @institution_document.institution.id)
    end
    authorize!(:destroy, @institution_document)
    @institution_document.all_children.each do |doc|
      authorize!(:destroy, doc)
    end
    InstitutionDocument.delete_all(id: ([@institution_document.id] + @institution_document.all_children.pluck(:id).to_a))
    respond_to do |format|
      format.html { redirect_to redirect_target, success: 'Eintrag wurde gelöscht' }
    end
  end

  private
  # Use callbacks to share common setup or constraints between actions.
  def find_institution_document
    InstitutionDocument.where(id: params[:id]).take!
  end

  def find_institution
    Institution.where(id: params[:institution_id]).take!
  end

  # Never trust parameters from the scary internet, only allow the white list through.
  def institution_document_params
    params.require(:institution_document).permit(
      :id,:file, :institution_id, :parent_id, :description, :step_type)
  end
end
