class PortfolioDiagnosticInformalResultsController < ApplicationController

  def new
    @portfolio_diagnostic_informal_result = ::Portfolio::DiagnosticInformalResult.new(
        diagnostic_result_acceptor_id: params[:diagnostic_result_acceptor_id],
        diagnostic_result_acceptor_type: params[:diagnostic_result_acceptor_type]
    )
    @portfolio_diagnostic_informal_result.build_formal_results
    @diagnostic_result_acceptor = @portfolio_diagnostic_informal_result.diagnostic_result_acceptor
    authorize!(:update, @diagnostic_result_acceptor.portfolio_item)
    subjects_breadcrumb(@diagnostic_result_acceptor.portfolio_item)
    subject_breadcrumb(@diagnostic_result_acceptor.portfolio_item)
    add_breadcrumb "#{@diagnostic_result_acceptor.portfolio_item.portfolio_step.name}", portfolio_item_path(@diagnostic_result_acceptor.portfolio_item)
    add_breadcrumb  "Ergebnis eintragen für #{@diagnostic_result_acceptor.name} vom #{l(@diagnostic_result_acceptor.start, format: :short)}"
  end

  def edit
    @portfolio_diagnostic_informal_result = ::Portfolio::DiagnosticInformalResult.where(id: params[:id]).take!
    @diagnostic_result_acceptor = @portfolio_diagnostic_informal_result.diagnostic_result_acceptor
    authorize!(:update, @diagnostic_result_acceptor.portfolio_item)
    subjects_breadcrumb(@diagnostic_result_acceptor.portfolio_item)
    subject_breadcrumb(@diagnostic_result_acceptor.portfolio_item)
    add_breadcrumb "#{@diagnostic_result_acceptor.portfolio_item.portfolio_step.name}", portfolio_item_path(@diagnostic_result_acceptor.portfolio_item)
    add_breadcrumb  "Ergebnis eintragen für #{@diagnostic_result_acceptor.name} vom #{l(@diagnostic_result_acceptor.start, format: :short)}"
  end

  def create
    @portfolio_diagnostic_informal_result = ::Portfolio::DiagnosticInformalResult.new
    @portfolio_diagnostic_informal_result.build_formal_results
    @portfolio_diagnostic_informal_result.assign_attributes(portfolio_diagnostic_informal_result_params)
    @diagnostic_result_acceptor = @portfolio_diagnostic_informal_result.diagnostic_result_acceptor
    authorize!(:update, @diagnostic_result_acceptor.portfolio_item)
    subjects_breadcrumb(@diagnostic_result_acceptor.portfolio_item)
    subject_breadcrumb(@diagnostic_result_acceptor.portfolio_item)
    add_breadcrumb "#{@diagnostic_result_acceptor.portfolio_item.portfolio_step.name}", portfolio_item_path(@diagnostic_result_acceptor.portfolio_item)
    add_breadcrumb  "Ergebnis eintragen für #{@diagnostic_result_acceptor.name} vom #{l(@diagnostic_result_acceptor.start, format: :short)}"
    respond_to do |format|
      if  @portfolio_diagnostic_informal_result.save
        format.html { redirect_to @portfolio_diagnostic_informal_result.diagnostic_result_acceptor.portfolio_item.portfolio_subject.subject_path,  success:  "Gespeichert."}
      else
        flash.now[:danger] = "Bitte prüfen Sie Ihre Eingaben"
        format.html { render :new }
      end
    end
  end

  def update
    @portfolio_diagnostic_informal_result = ::Portfolio::DiagnosticInformalResult.where(id: params[:id]).take!
    @diagnostic_result_acceptor = @portfolio_diagnostic_informal_result.diagnostic_result_acceptor
    authorize!(:update, @diagnostic_result_acceptor.portfolio_item)
    @portfolio_diagnostic_informal_result.assign_attributes(portfolio_diagnostic_informal_result_params)
    subjects_breadcrumb(@diagnostic_result_acceptor.portfolio_item)
    subject_breadcrumb(@diagnostic_result_acceptor.portfolio_item)
    add_breadcrumb "#{@diagnostic_result_acceptor.portfolio_item.portfolio_step.name}", portfolio_item_path(@diagnostic_result_acceptor.portfolio_item)
    add_breadcrumb  "Ergebnisse eintragen für #{@diagnostic_result_acceptor.name} vom #{l(@diagnostic_result_acceptor.start, format: :short)}"
    respond_to do |format|
      if  @portfolio_diagnostic_informal_result.save
        format.html { redirect_to @portfolio_diagnostic_informal_result.diagnostic_result_acceptor.portfolio_item.portfolio_subject.subject_path,  success:  "Gespeichert."}
      else
        flash.now[:danger] = 'Bitte prüfen Sie Ihre Eingaben'
        format.html { render :edit }
      end
    end
  end

  def destroy
    @portfolio_diagnostic_informal_result = ::Portfolio::DiagnosticInformalResult.where(id: params[:id]).take!
    acceptor = @portfolio_diagnostic_informal_result.diagnostic_result_acceptor
    portfolio_item = acceptor.portfolio_item
    authorize!(:destroy, portfolio_item)
    respond_to do |format|
      if @portfolio_diagnostic_informal_result.can_be_deleted? && @portfolio_diagnostic_informal_result.destroy
        format.html { redirect_to portfolio_item_path(portfolio_item),  success:  "Ergebnis gelöscht." }
      else
        format.html { redirect_to portfolio_item_path(portfolio_item),  danger:  "Das Ergebnis kann nicht gelöscht werden. #{@portfolio_diagnostic_informal_result.cannot_be_deleted_reason}"  }
      end
    end
  end


  private
    def portfolio_diagnostic_informal_result_params
      params.require(:portfolio_diagnostic_informal_result).permit(
          :id, :result, :diagnostic_result_acceptor_id, :diagnostic_result_acceptor_type, :subject_involved,
          involved_teacher_ids: [],
          involved_guardian_ids: [],
          involved_supporter_ids: [],
          involved_people_attributes: [
              :id,:name,:_destroy
          ],
          results_attributes: [
             :id,:characteristic_id,:norm_scale_id,:result
          ],
          documents_attributes: [
              :id,:file_name, :file_name_cache, :_destroy
          ]
      )
    end


    def subjects_breadcrumb(portfolio_item)
      case portfolio_item.portfolio.subject.class.name
        when 'Client'
          add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
        when 'ClientGroup'
          add_breadcrumb "Einrichtung", institution_path(portfolio_item.portfolio_subject.institution), title: "#{portfolio_item.portfolio_subject.institution.name}"
      end

    end

    def subject_breadcrumb(portfolio_item)
      case portfolio_item.portfolio.subject.class.name
        when 'Client'
          add_breadcrumb "#{view_context.show_person(portfolio_item.portfolio.subject)}", dashboard_client_path(portfolio_item.portfolio.subject), title: "Übersicht anzeigen"
        when 'ClientGroup'
          add_breadcrumb "Klasse #{portfolio_item.portfolio_subject.name}", client_group_path(portfolio_item.portfolio.subject), title: "Klassenübersicht anzeigen"
      end
    end

end
