# Enumfilter only shows the IDS when filter_value_as_string is called.
class Datagrid::Filters::EnumFilter
  def format(value)
    found =select.find do |element|
      if element.is_a? Array
        element.last.to_s == value
      else
        element.to_s == value
      end
    end
    if found.is_a? Array
      return found.first
    else
      return found
    end
  end
end

require 'digest/md5'
class BaseGrid
  include Datagrid

  # get only the filters that have any non blank value set
  def active_filters
    filters.select{ |f| self.filter_value(f).present? }
  end


  # when defining a column, enable a header row when that column is sorted by
  #
  # use option :group_header => Proc to define the content of the header row
  # use option :group_header_collapse => 'out' to make the following rows collapsible, initially open
  # use option :group_header_collapse => 'in' to make the following rows collapsible, initially collapsed

  # Example:
  #
  # column(:class, header: 'Klasse', html: true, order: :client_group_id, group_header_collapse: 'in',
  #   group_header: Proc.new {|c| c.client_group ? "Klasse #{c.client_group.name}" : "Keine Klasse" }) do |c|
  #   show_client_group(c.client_group)
  # end
  #
  def print_group_header_column(asset)
    @group_header_column_last_value = {} if @group_header_column_last_value.nil?
    columns.each do |column|
      if column.options[:group_header]
        if ordered_by?(column.name)
          value = column.options[:group_header].call(asset)
          unless @group_header_column_last_value.key?(column.name)
            if column.options[:group_header_collapse]
              @current_header_grouping_id = "#{column.name}_#{Digest::MD5.hexdigest(value)}"
              @current_header_grouping_classes = "collapse #{column.options[:group_header_collapse]} #{@current_header_grouping_id}"
            end
            @group_header_column_last_value[column.name] = value
            yield column, value
          end
          if @group_header_column_last_value[column.name] != value
            @group_header_column_last_value[column.name] = value
            if column.options[:group_header_collapse]
              @current_header_grouping_id = "#{column.name}_#{Digest::MD5.hexdigest(value)}"
              @current_header_grouping_classes = "collapse #{column.options[:group_header_collapse]} #{@current_header_grouping_id}"
            end
            yield column, value
          end
        end
      end
    end
  end

  # this is used in datagrid/_row.html.erb
  def current_header_grouping_id
    @current_header_grouping_id
  end

  # this is used in datagrid/_row.html.erb
  def current_header_grouping_classes
     @current_header_grouping_classes
  end




  # get the complete query for this report WITHOUT the filter passed
  # this is to be used when a search filter has to be discarded (by clicking appropriate button)
  #
  # Example:
  # params.to_h -> {:page=>5, :model_grid => {:param1 => 'value1', :param2 => 'value2' } }
  # filter.name.to_s -> "param_1"
  # exclude_filter_params(filter) -> {:model_grid => { :param2 => 'value2' } }
  #
  # Note that only the grid parameters (order,filter) are preserved, all other request parameters (like page) are
  # not included in the returend hash
  #
  def exclude_filter_params(filter)
    _params = self.as_query.map{|k,v| k == filter.name ? [k, ''] : [k, v]  }.to_h
     { self.param_name => _params }
  end

  def all_asset_ids
    if @all_asset_ids.blank?
      @all_asset_ids = assets.map{|a| a.id }
    end
    @all_asset_ids
  end

end