class BatchImporter
  include ActiveModel
  # teacher.name, teacher.relation1.name, teacher.relation2.name, teacher.relation2.subrel.test, teacher.relation2.subrel.toast
  # X           , b                     , c                     , 3                            , NULL
  # NULL        , d                     , NULL                  , NULL                         , NULL
  # NULL        , NULL                  , e                     , NULL                         , NULL
  # NULL        , NULL                  , NULL                  , NULL                         , medium
  # Y           , b                     , c                     , 3                            , NULL
  #{ teacher: { name: 'X', relation1: { name:'b' }, relation2: { name:'c', subrel: { test: 3,   toast: nil } } } }
  #{ teacher: { name: nil, relation1: { name:'d' }, relation2: { name:nil, subrel: { test: nil, toast: nil } } } }
  #{ teacher: { name: nil, relation1: { name:nil }, relation2: { name:'e', subrel: { test: nil, toast: nil } } } }
  #{ teacher: { name: nil, relation1: { name:nil }, relation2: { name:nil, subrel: { test: nil, toast: 'medium' } } } }
  #{ teacher: { name: 'Y', relation1: { name:'b' }, relation2: { name:'c', subrel: { test: 3,   toast: nil } } } }
  def initialize
    @messages = []
    @model_collector_class=ModelCollector
  end
  def add_message(m)
    @messages << m
  end


  def available_fields
    @model_collector_class.available_fields
  end

  def import(csv,dry_run=false,debug=false)
    @messages=[]
    model_collector = @model_collector_class.new(debug)
    CSV.foreach(csv, headers: true) do |row|
      model_collector.line_number+=1
      model_collector.collect(row)
      model_collector.flush_messages.each do |message|
        add_message message
      end
      ActiveRecord::Base.transaction do
        model_collector.flush_models.each do |model|
          if dry_run
            if model.validate
              add_message "Line #{model.__batch_importer_line_number}: WILL CREATE  #{model.class.name} WITH ATTRIBUTES=#{model.attributes.inspect}" if model.new_record?
              add_message "Line #{model.__batch_importer_line_number}: WILL UPDATE  #{model.class.name}##{model.id} WITH ATTRIBUTES=#{model.attributes.inspect}" if model.persisted?
            else
              add_message "Line #{model.__batch_importer_line_number}: INVALID #{model.class.name} VALIDATION_ERRORS=#{model.errors.messages.inspect}"
            end
          else
            if model.save
              add_message "Line #{model.__batch_importer_line_number}: CREATE  #{model.class.name}" if model.new_record?
              add_message "Line #{model.__batch_importer_line_number}: UPDATE  #{model.class.name}##{model.id}" if model.persisted?
            else
              add_message "Line #{model.__batch_importer_line_number}: INVALID  #{model.class.name} VALIDATION_ERRORS=#{model.errors.messages.inspect}"
              raise ActiveRecord::Rollback
            end
          end
        end
      end
    end
    ActiveRecord::Base.transaction do
      model_collector.flush_resolved.each do |model|
        if dry_run
          if model.validate
            add_message "Line #{model.__batch_importer_line_number}: WILL CREATE  #{model.class.name} WITH ATTRIBUTES=#{model.attributes.inspect}" if model.new_record?
            add_message "Line #{model.__batch_importer_line_number}: WILL UPDATE  #{model.class.name}##{model.id} WITH ATTRIBUTES=#{model.attributes.inspect}" if model.persisted?
          else
            add_message "Line #{model.__batch_importer_line_number}: INVALID #{model.class.name} VALIDATION_ERRORS=#{model.errors.messages.inspect}"
          end
        else
          if model.save
            add_message "Line #{model.__batch_importer_line_number}: CREATE  #{model.class.name}" if model.new_record?
            add_message "Line #{model.__batch_importer_line_number}: UPDATE  #{model.class.name}##{model.id}" if model.persisted?
          else
            add_message "Line #{model.__batch_importer_line_number}: INVALID  #{model.class.name} VALIDATION_ERRORS=#{model.errors.messages.inspect}"
            raise ActiveRecord::Rollback
          end
        end
      end
    end
    return @messages
  end

  private
  class ModelCollector
    include ActiveModel
    attr_accessor :line_number
    def initialize(debug)
      @debug=debug
      @previous = {}
      @can_be_saved = {}
      @resolved_so_far =Set.new
      @messages = []
      @last_parent_model=nil
      @line_number=0
    end

    def collect(row,parent_model=nil,root=true)
      simple_attributes = {}
      sub_rows={}
      row.each do |key,value|
        parts = key.split('.')

        token=parts.shift
        simple_attributes[token] ||= {}
        simple_attributes[token][parts.first] = value if parts.size==1
        sub_rows[token] ||= {}
        sub_rows[token][parts.join('.')] = value if parts.size>1
      end

      simple_attributes.each do |token,attributes|
        parent_model = "::#{token.camelize}".constantize.new if parent_model.nil? && root
        next if parent_model.nil?
        if attributes.values.all?(&:blank?)
          model_instance = get_previous_instance(parent_model,token)
        else
          if root
            @can_be_saved[token] ||= []
            @can_be_saved[token] = flush_resolved
          end
          model_instance = resolve( parent_model,token,attributes)
          @resolved_so_far.add(model_instance)  unless model_instance.nil?
        end
        collect(::CSV::Row.new(sub_rows[token].keys,sub_rows[token].values,true),model_instance,false)
      end
    end



    def get_model_helper(parent_model)
       return ModelHelperBase.new(parent_model) # this helper does not allow any deeper relations
    end

    def self.available_fields
      %w()
    end

    def flush_resolved
      r = @resolved_so_far.to_a
      @resolved_so_far = Set.new
      return r
    end

    def flush_models
      r = @can_be_saved.values.flatten
      @can_be_saved = {}
      return r
    end

    def flush_messages
      c = @messages
      @messages = []
      return c
    end


    def add_message(m)
      @messages << "Line #{line_number}: #{m}"
    end

    def resolve(parent_model,token,attributes)
      begin
        model_helper = get_model_helper(parent_model)
        model_helper.debug = @debug
        delete_previous_instance(parent_model,token)
        model_instance = model_helper.query(token,attributes)
        unless model_instance.nil?
          class << model_instance
            attr_accessor :__batch_importer_line_number
          end
          model_instance.__batch_importer_line_number=@line_number
        end
        set_previous_instance(parent_model,token,model_instance)
        model_helper.populate(token,attributes,model_instance)
        model_helper.messages.each do |message|
          add_message message
        end
      rescue ModelHelperException => e
        add_message e.message
        return nil
      end
      return model_instance
    end

    def delete_previous_instance(parent_model,token)
      r=@previous.delete("#{parent_model.class.name}.#{token}")
      r
    end

    def get_previous_instance(parent_model,token)
      r=@previous["#{parent_model.class.name}.#{token}"]
      r
    end

    def set_previous_instance(parent_model,token,model_instance)
      @previous["#{parent_model.class.name}.#{token}"] = model_instance
    end

  end

  class ModelHelperException < ::StandardError
  end

  class ModelHelperBase
    attr_reader :messages
    attr_accessor :debug
    def initialize(model)
      @model=model
      @messages=[]
      @definitions={}
    end
    def add_message(token,message)
      @messages << " #{self.class.name}.#{token} on model #{show_model(@model)} #{message}" if @debug
    end
    def show_model(model)
      "#{model.class.name}##{model.object_id.to_s.last(4)}"
    end
    def query(token,attributes)
      result = execute_query(token,attributes)
      if result.blank? || (result.respond_to?(:size) && result.size == 0) # is no result
        result = create(token,attributes) # ggfs erzeugen und am parent setzen
        add_message token, "create #{show_model(result)} by #{attributes.inspect}"
      elsif result.is_a? ActiveRecord::Base # is single result, leave model instance as it is
        add_message token, "found #{show_model(result)} by #{attributes.inspect}"
      elsif (result.respond_to?(:size) && result.size == 1) # is single result
        result = result.first
        add_message token, "found #{show_model(result)} by #{attributes.inspect}"
      else # is multi result or sth else we cannot use
        raise ModelHelperException.new("Too many results for #{token} with #{attributes} on #{@model.class.name}")
      end
      result
    end
    def create(token,attributes)
      attributes = attributes.select { |name,value| allow_attribute(token,name,value) }
      return nil if attributes.empty?
      result = execute_create(token, attributes)
      if result.nil?
        raise ModelHelperException.new(add_message token, "Cannot create with #{attributes}")
      end
      result
    end
    def populate(token, attributes, instance)
      attributes = attributes.select { |name,value| allow_attribute(token,name,value) }
      add_message token, "populating with #{attributes.inspect}"
      instance.assign_attributes(attributes) unless attributes.empty?
    end
    private
    def execute_query(token,attributes)
      nil
    end
    def execute_create(token, attributes)
      nil
    end
    def allow_attribute(token,name,value)
      false
    end

  end

end

