module Portfolio
  # This takes a portfolio ( a collection of items ) and checks them against portfolio rules
  class Chain

    #attr_reader :portfolio,
    attr_reader :last_matching_item, :date
    delegate :documents, to: :@enumerator

    def initialize(portfolio_items,portfolio_rules,portfolio_level)
      @portfolio_level = portfolio_level
      @portfolio_rules = portfolio_rules.order(position: :desc).all
      @enumerator = ::Portfolio::Enumerator.new(portfolio_items)
      init_stacks
    end

    def run
      @date =  @enumerator.to_a.last.blank? ? Date.today : @enumerator.to_a.last.start
      if @enumerator.current_item.nil? || @enumerator.current_item.start > date
        @enumerator.rewind
        init_stacks
      end
      # scan forward to a rule with minimum level, in case a subject with higher than 0 level starts portfolio
      while @portfolio_level > next_level
        @fulfilled << @rules.pop
      end
      # forward the iterator as well
      @enumerator.forward_to_level(@portfolio_level)
      while @enumerator.next? && @enumerator.peek.start <= date
        @enumerator.next
        match if has_next_rule?
      end
    end


    # @return Array[::Portfolio::Step]
    def allowed_steps(level)
     return next_steps if level >= next_level
     return last_steps if level <= last_level
    end

    def block_level_raise?
      has_next_rule? && !next_rule.allow_level_raise(@enumerator)
    end

    def next_level
      return last_level unless has_next_rule?
      return next_rule.level
    end

    def last_level
      return 0 unless has_last_rule?
      return last_rule.level
    end

    def next_steps
      return last_steps unless has_next_rule?
      return next_rule.allowed_portfolio_steps_for(@enumerator)
    end

    def last_steps
      return [] unless has_last_rule?
      return last_rule.allowed_portfolio_steps_for(@enumerator)
    end

    def has_last_rule?
      @fulfilled.any?
    end

    def has_next_rule?
      @rules.any?
    end

    def last_rule
      return nil unless has_last_rule?
      @fulfilled.last
    end

    def next_rule
      return nil unless has_next_rule?
      @rules.last
    end


    private
    def init_stacks
      @fulfilled = []
      @rules = @portfolio_rules.to_a
    end

    def match
      matching_condition = next_rule.matching_condition(@enumerator)
      unless matching_condition.nil?
        @last_matching_item = @enumerator.current_item
        forward_rules(calculate_position(matching_condition))
      end
      return matching_condition.present?
    end

    def forward_rules(position)
      while has_last_rule? && ( ! has_next_rule? || next_rule.position > position) do
        @rules << @fulfilled.pop
      end
      while has_next_rule? && next_rule.position < position  do
        @fulfilled << @rules.pop
      end
    end

    def calculate_position(matching_condition)
      position = matching_condition.next_position
      if position.nil?
        position=@rules.last.position+1
      end
      return position
    end

  end
end
