module Portfolio
  module Subjects
    class Base

      def initialize(portfolio,subject)
        @portfolio=portfolio
        @subject=subject
      end

      def self.all_subjects
        ['Client','ClientGroup']
      end

      delegate :portfolio_level, to: :@subject
      delegate :institution, :institution_id, to: :@subject

      def client_group
        nil
      end

      def client_group_id
        nil
      end

      def name
        "#{@subject.class.name} #{@subject.id}"
      end

      # @return [Array<AdvancementPlan>]
      def available_remote_diagnostic_suggestions
        []
      end

      # @return [Array<AdvancementPlan>]
      def available_remote_advancement_plan_suggestions
        []
      end

      def relation_description_for_guardian(guardian)
        ''
      end

      # get the records that are allowed to be set as involved teachers for this portfolio
      # which are teachers at the same institution as the portfolio ( author is excluded, as it is set via checkbox)
      #
      # this is used for the select involved teachers field in portfolio item create/update form
      # @return [ActiveRecord::Associations::CollectionProxy<::Teacher>]
      def portfolio_item_available_involved_teachers(portfolio_item)
        _records = ::Teacher.joins(:teacher_institution_relations)
                       .where('teacher_institution_relations.institution_id=?',institution_id)
        _records = _records.where('teachers.id <> ?',portfolio_item.author.id) if portfolio_item.author.class.name == 'Teacher'
        return _records
      end

      # get the records that are  allowed to be set as involved guardians for this portfolio
      #
      # for base items, guardians of clients at the same institution as the portfolio are returned
      # this is used for the select involved guardians field in portfolio item create/update form
      #
      # @return [ActiveRecord::Associations::CollectionProxy<::Guardian>]
      def portfolio_item_available_involved_guardians(portfolio_item)
        ::Guardian.none
      end


      # get the records that are allowed to be set as involved supporters
      # which are supporters related all clients in the same institution as the portfolio
      #
      # this is used by the select involved supporters field in portfolio item create/update form
      # @return [ActiveRecord::Associations::CollectionProxy<::Supporter>]
      def portfolio_item_available_involved_supporters(portfolio_item)
        ::Supporter.none
      end

      # get all portfolio specializations that can be set for the item
      #
      # this is used by the select portfolio_specialization field in portfolio item create/update form
      # @return [ActiveRecord::Associations::CollectionProxy<::PortfolioSpecialization>]
      def available_specializations(portfolio_item)
        ::PortfolioSpecialization.all
      end

      # get the records that are allowed to be set for relation
      #
      # @return [Array<String>]
      def evaluation_available_result_schemes(portfolio_item_evaluation)
        ::Portfolio::ItemTargetResult.result_schemes
      end


      #  get the records that are  allowed to be set as involved guardians
      #
      # this is used by the select field for evaluation involved teachers in portfolio item create/update form
      #
      # @return [ActiveRecord::Associations::CollectionProxy<::Guardian>]
      def evaluation_available_involved_guardians(portfolio_item_evaluation)
        ::Guardian.none
      end

      # get the records that are allowed to be set as involved teachers
      # which are teachers at the same institution as the client
      #
      # this is used by the select field for evaluation involved teachers  in portfolio item create/update form
      # @return [ActiveRecord::Associations::CollectionProxy<::Teacher>]
      def evaluation_available_involved_teachers(portfolio_item_evaluation)
        _records = ::Teacher.joins(:teacher_institution_relations).
            where('teacher_institution_relations.institution_id=? OR teachers.id IN (?)',institution_id,portfolio_item_evaluation.involved_teachers.ids)
        return _records.distinct(:id)
      end

      # get the records that are allowed to be set as involved supporters
      # which are supporters related to the client
      #
      # this is used by the select field for evaluation involved supporters  in portfolio item create/update form
      # @return [ActiveRecord::Associations::CollectionProxy<::Supporter>]
      def evaluation_available_involved_supporters(portfolio_item_evaluation)
        ::Supporter.none
      end

      def allows_guardians?(portfolio_item)
        true
      end


      def is_class_teacher?(teacher)
        false
      end

      def is_support_teacher?(teacher)
        teacher.teacher_category.present? && teacher.teacher_category.is_support_teacher?
      end

      def is_special_ed?(teacher)
        teacher.teacher_category.present? && teacher.teacher_category.is_special_ed?
      end


      def has_special_ed?
        false
      end

      def is_management?(teacher)
        teacher.teacher_institution_relations.select{ |r| r.institution_id == institution_id && r.teacher_institution_category.try{is_management?} }.any?
      end

      def subject_path
        '/'
      end

      def resolve_notifications_path
        '/'
      end

      def select_portfolio_item_date_path
        '/'
      end

      def redirect_after_item_create
        '/'
      end

      def portfolio_title
        'Portfolio'
      end

    end
  end
end