module SystemTasks
  class BorrowAdvancementPlan < Base

    def _escalations
      [
          {
            Escalator::SendUserMessage => [target.portfolio_item.author,target.portfolio_item.editor]
          },
          {
            Escalator::SendUserMessage => target.portfolio_item.involved_teachers + target.portfolio_item.evaluable_target_agreements.inject([]) do |memo,i|
              memo + i.evaluation.involved_teachers
            end
          }
      ]
    end

    def is_manual_confirmable?
      true
    end

    class << self
      def subscribe_to
        [
            {
                klass: 'portfolio/item_advancement_plan',
                callback: 'after_create',
                handler: 'portfolio_item_advancement_plan_after_create'
            }
        ]
      end
      def portfolio_item_advancement_plan_after_create(portfolio_item_advancement_plan)
        create({
             target: portfolio_item_advancement_plan,
             client: (portfolio_item_advancement_plan.portfolio.subject.is_a? Client) ? portfolio_item_advancement_plan.portfolio.subject : nil,
             institution: (portfolio_item_advancement_plan.portfolio.subject.is_a? ClientGroup) ? portfolio_item_advancement_plan.portfolio.subject_proxy.institution : nil,
             start_at: DateTime.now,
             label: portfolio_item_advancement_plan.portfolio.subject_proxy.name,
             title:  "Fördermethode #{portfolio_item_advancement_plan.advancement_plan.title} ausleihen?",
             message: "Die Fördermethode #{portfolio_item_advancement_plan.advancement_plan.title} für den Eintrag vom #{I18n::l(portfolio_item_advancement_plan.portfolio_item.start,format: :short)} für #{portfolio_item_advancement_plan.portfolio.subject_proxy.name} kann ausgeliehen werden.",
             due_at: portfolio_item_advancement_plan.begin,
             next_escalation: DateTime.now,
             escalation_delay: 86400,
             resolve_location: Rails.application.routes.url_helpers.advancement_plan_path(catalog_id: portfolio_item_advancement_plan.catalog_id)
         })
      end
    end

  end
end