module SystemTasks
  class ParentTeacherConference < Base
    def _escalations
      if target.subject.is_a? Client
        return [
            {
                Escalator::SendUserMessage => [target.subject.teacher, target.subject.try(:client_group).try(:teacher)]
            },
            {
                Escalator::SendUserMessage => target.subject.institution.teacher_institution_relations.where(is_school_admin: true).all.to_a
            }
        ]
      end
      if target.subject.is_a? ClientGroup
        return [
            {
                Escalator::SendUserMessage => [target.subject.teacher]
            },
            {
                Escalator::SendUserMessage => target.subject.institution.teacher_institution_relations.where(is_school_admin: true).all.to_a
            }
        ]
      end
    end

    def is_manual_confirmable?
      false
    end

    class << self
      def subscribe_to
        [
            {
                klass: 'portfolio/item',
                callback: 'before_save',
                handler: 'portfolio_item_change'
            },
            {
                klass: 'portfolio/item',
                callback: 'after_destroy',
                handler: 'portfolio_item_change'
            }
        ]
      end


      def portfolio_item_change(portfolio_item)
        if (portfolio_item.portfolio_step.allowed?(:parent_teacher_conference) && (portfolio_item.parent_teacher_conference_changed? || portfolio_item.destroyed?)) || portfolio_item.portfolio_step.type == 'parent_teacher_conference'
          Rails.logger.debug  "PT_CONF: begin"
          conference_required = false
          last_requiring_item = nil
          last_fulfilling_item = nil
          enumerator = ::Portfolio::Enumerator.new(portfolio_item.portfolio.portfolio_items)
          enumerator.each do |item|
            Rails.logger.debug  "PT_CONF: inspecting item <#{item.id}>"
            if portfolio_item.portfolio_step.type == 'parent_teacher_conference'
              conference_required = false
              last_fulfilling_item = item
              Rails.logger.debug  "PT_CONF: found last fulfilling item <#{item.id}>"
            end
            if item.parent_teacher_conference?
              conference_required = true
              last_requiring_item = item
              Rails.logger.debug  "PT_CONF: found last requiring item <#{item.inspect}>"
            end
          end
          if conference_required && last_requiring_item
            Rails.logger.debug  "PT_CONF: conf required by item <#{last_requiring_item.id}>"
            unless where(target: last_requiring_item.portfolio).exists?
              Rails.logger.debug  "PT_CONF: could not find existing task for portfolio <#{last_requiring_item.portfolio.id}>"
              task = create!({
                         target: last_requiring_item.portfolio,
                         client: (last_requiring_item.portfolio.subject.is_a? Client) ? last_requiring_item.portfolio.subject : nil,
                         institution: (last_requiring_item.portfolio.subject.is_a? ClientGroup) ? last_requiring_item.portfolio.subject_proxy.institution : nil,
                         label: last_requiring_item.portfolio.subject_proxy.name,
                         title: "Ein Elterngespräch für #{last_requiring_item.portfolio.subject_proxy.name} ist angesetzt.",
                         message: ::I18n.translate('system_notifications.parent_teacher_conference.content',
                                                   step: last_requiring_item.portfolio.subject_proxy.name,
                                                   step_date: ::I18n.localize(last_requiring_item.start, format: :short),
                                                   parent_teacher_conference_date: ::I18n.localize(last_requiring_item.parent_teacher_conference_date, format: :short),
                         ),
                         start_at: last_requiring_item.start,
                         due_at: last_requiring_item.parent_teacher_conference_date,
                         next_escalation: last_requiring_item.start,
                         escalation_delay: 86400 * 7,
                         resolve_location: last_requiring_item.portfolio.subject_proxy.resolve_notifications_path
                     })
              Rails.logger.debug  "PT_CONF: created task <#{task.id}>"
            end
          else
            Rails.logger.debug  "PT_CONF: conference not required"
            if last_fulfilling_item
              Rails.logger.debug  "PT_CONF: deleting for portfolio <#{last_fulfilling_item.portfolio.id}>"
              where(target: last_fulfilling_item.portfolio).delete_all
            end
          end
        end
      end


    end
  end
end