module SystemTasks
  module PortfolioRuleNotifications
    class Base < SystemTasks::Base

      def _escalations
        if target.subject.is_a? Client
          return  [
              {
                  Escalator::SendUserMessage => [target.subject.try(:teacher),target.subject.try(:client_group).try(:teacher)]
              },
              {
                  Escalator::SendUserMessage => target.subject.institution.teacher_institution_relations.where(is_school_admin:true).all.to_a
              }
          ]
        end
        if target.subject.is_a? ClientGroup
          return  [
              {
                  Escalator::SendUserMessage => [target.subject.try(:teacher)]
              },
              {
                  Escalator::SendUserMessage => target.subject.institution.teacher_institution_relations.where(is_school_admin:true).all.to_a
              }
          ]
        end
      end

      def is_manual_confirmable?
        false
      end

      class << self
        def subscribe_to
          [
              {
                  klass: 'portfolio/item',
                  callback: 'after_save',
                  handler: 'portfolio_item_after_save'
              },
              {
                  klass: 'portfolio/item',
                  callback: 'after_destroy',
                  handler: 'portfolio_item_after_destroy'
              },
              {
                  klass: 'career_item',
                  callback: 'after_create',
                  handler: 'career_item_after_create'
              }
          ]
        end

        # @param portfolio_item ::Portfolio::Item
        def portfolio_item_after_save(portfolio_item)
          if portfolio_item.closed?
            apply_rule_notifications(portfolio_item.portfolio)
          end
        end


        # @param portfolio_item ::Portfolio::Item
        def portfolio_item_after_destroy(portfolio_item)
          if portfolio_item.closed?
            apply_rule_notifications(portfolio_item.portfolio)
          end
        end

        # @param career_item ::CareerItem
        def career_item_after_create(career_item)
          return unless career_item[:change_type] == CareerItem::CHANGE_TYPE_LEVEL
          apply_rule_notifications(career_item.client.running_portfolio)
        end

        def apply_rule_notifications(portfolio)
          notifiers = portfolio.next_rule_notifiers.select{|notifier| notifier.class.name.demodulize == name.demodulize}
          if notifiers.blank?
            where(target: portfolio).delete_all
          else
            notifiers.each do |notifier|
              if notifier.show? && !where(target: portfolio).exists?
                do_create(notifier)
              else
                where(target: portfolio).delete_all
              end
            end
          end
          true
        end

        def do_create(notifier)
          raise ::StandardError.new('Implementation missing')
        end
      end

    end
  end
end