module CsbApi

  # Represents the csb api resultset type "institutions"
  #
  class Institution < CsbApi::Base
    def self.table_name; "institutions"; end

    # populate institution activerecord instance from this result
    #
    # When institution instance is omitted, a new instance is returned
    # When institution instance is set, it is populated and returned
    #
    # @param model [Institution] the instance to populate
    # @return [Institution] the populated instance
    def build_model(model = ::Institution.new)
      model.attributes={
          name: read_attribute('name'),
          school_no: read_attribute('school-no'),
          email: read_attribute('email'),
          fax: read_attribute('fax'),
          website: read_attribute('website'),
          description: read_attribute('description'),
          #allocation_formula: read_attribute('allocation-formula'),
          comment: read_attribute('comment'),
          phone: read_attribute('phone'),
      }
      if relationships['address'].key? "data" # the records are included in the result set
        remote = self.send('address')
        if model.address
          remote.populate_model(model.address)
        else
          model.address = remote.build_model
        end
      end
      if relationships['institution-category'].key? "data" # the records are included in the result set
        remote = self.send('institution-category')
        category = remote.build_model
        local_category = ::InstitutionCategory.where(vid: category.vid, region: category.region).take
        if local_category.present?
          model.institution_category = local_category # in this place, the category is only set if it already exists locally. When not, it is left empty
        end
      end
      return model
    end
    alias :populate_model :build_model

    # Represents the csb api resultset type "institution address"
    #
    class Address < CsbApi::Base
      def self.table_name; "addresses"; end
      # populate ar instance from this result
      #
      # @param model [Address] the instance to populate
      # @return [Address] the populated instance
      def build_model(model = ::Address.new)
        model.attributes = {
            street: read_attribute('street'),
            number: read_attribute('number'),
            city: read_attribute('city'),
            region: read_attribute('region'),
            zip: read_attribute('zip'),
            country: read_attribute('country'),
            latitude: read_attribute('latitude'),
            longitude: read_attribute('longitude')
        }
        return model
      end
      alias :populate_model :build_model
    end

    # Represents the csb api resultset type "institution address"
    #
    class InstitutionCategory < CsbApi::Base
      def self.table_name; "institution-categories"; end
      # populate ar instance from this result
      #
      # @param model [InstitutionCategory] the instance to populate
      # @return [InstitutionCategory] the populated instance
      def build_model(model = ::InstitutionCategory.new)
        model.attributes = {
            vid: read_attribute('vid'),
            region: read_attribute('region')
        }
        return model
      end
      alias :populate_model :build_model
    end
  end
end
