module VsbApi
  module V1
    class SimilarClient < ActiveRecord::Base
    end
    class SimilarClientProcessor < JSONAPI::Processor
      class << self
        def transactional_operation_type?(operation_type)
          true # we need a transaction in any case as find results are temporary stored in ON COMMIT DROP memory table
        end
      end
      before_find do |arg|
        filters =  arg.params[:filters]
        ActiveRecord::Base.connection.execute('DROP TABLE IF EXISTS similar_clients ; CREATE TEMPORARY TABLE  similar_clients
          (
          id integer,
          client_first_name VARCHAR,
          client_last_name VARCHAR,
          client_bday VARCHAR,
          client_gender VARCHAR,
          client_gender_name VARCHAR,
          similar_client_id VARCHAR,
          similar_client_first_name VARCHAR,
          similar_client_last_name VARCHAR,
          similar_client_bday VARCHAR,
          similar_client_gender VARCHAR,
          similar_client_gender_name VARCHAR,
          similar_client_institution_id VARCHAR,
          similar_client_institution_name VARCHAR,
          similar_client_institution_school_no VARCHAR,
          similar_client_institution_email VARCHAR,
          similar_client_institution_fax VARCHAR,
          similar_client_institution_website VARCHAR,
          similar_client_institution_phone VARCHAR,
          similar_client_institution_address_street VARCHAR,
          similar_client_institution_address_city VARCHAR,
          similar_client_institution_address_region VARCHAR,
          similar_client_institution_address_zip VARCHAR,
          similar_client_institution_address_country VARCHAR,
          similar_client_institution_address_number VARCHAR
          )
          ON COMMIT  DROP')
        if filters.key?(:client_last_name) && filters.key?(:client_last_name) && filters.key?(:client_bday)
          Client.duplicates(filters[:client_first_name],filters[:client_last_name],filters[:client_bday]).each_with_index do |record,i|
            SimilarClient.connection.execute("INSERT INTO similar_clients ( id,
                  client_first_name,
                  client_last_name,
                  client_bday,
                  client_gender,
                  client_gender_name,
                  similar_client_id,
                  similar_client_first_name,
                  similar_client_last_name,
                  similar_client_bday,
                  similar_client_gender,
                  similar_client_gender_name,
                  similar_client_institution_id,
                  similar_client_institution_name,
                  similar_client_institution_school_no,
                  similar_client_institution_email,
                  similar_client_institution_fax,
                  similar_client_institution_website,
                  similar_client_institution_phone,
                  similar_client_institution_address_street,
                  similar_client_institution_address_city,
                  similar_client_institution_address_region,
                  similar_client_institution_address_zip,
                  similar_client_institution_address_country,
                  similar_client_institution_address_number) VALUES
                  (#{i},
                  #{SimilarClient.connection.quote( filters[:client_first_name].to_s )} ,
                  #{SimilarClient.connection.quote( filters[:client_last_name].to_s )},
                  #{SimilarClient.connection.quote( filters[:client_bday].to_s )},
                  #{SimilarClient.connection.quote( filters[:client_gender].to_s )},
                  #{SimilarClient.connection.quote( Client.genders[filters[:client_gender]].to_s )},
                  #{SimilarClient.connection.quote(record.id.to_s )},
                  #{SimilarClient.connection.quote(record.first_name.to_s )},
                  #{SimilarClient.connection.quote( record.last_name.to_s )},
                  #{SimilarClient.connection.quote( record.bday.to_s )},
                  #{SimilarClient.connection.quote( Client.genders[record.gender].to_s )},
                  #{SimilarClient.connection.quote( record.gender )},
                  #{SimilarClient.connection.quote( record.institution.id.to_s )},
                  #{SimilarClient.connection.quote( record.institution.name.to_s )},
                  #{SimilarClient.connection.quote( record.institution.school_no.to_s )},
                  #{SimilarClient.connection.quote( record.institution.email.to_s )},
                  #{SimilarClient.connection.quote( record.institution.fax.to_s )},
                  #{SimilarClient.connection.quote( record.institution.website.to_s )},
                  #{SimilarClient.connection.quote( record.institution.phone.to_s )},
                  #{SimilarClient.connection.quote( record.institution.address.street.to_s )},
                  #{SimilarClient.connection.quote( record.institution.address.city.to_s )},
                  #{SimilarClient.connection.quote( record.institution.address.region).to_s },
                  #{SimilarClient.connection.quote( record.institution.address.zip.to_s )},
                  #{SimilarClient.connection.quote( record.institution.address.country.to_s )},
                  #{SimilarClient.connection.quote( record.institution.address.number.to_s )}
                  )" )
          end
        end
      end
    end
    class SimilarClientResource < JSONAPI::Resource
      model_name 'VsbApi::V1::SimilarClient'; def self._model_class; VsbApi::V1::SimilarClient; end
      paginator :none

      attribute :client_first_name
      attribute :client_last_name
      attribute :client_bday
      attribute :client_gender
      attribute :client_gender_name
      attribute :similar_client_id
      attribute :similar_client_first_name
      attribute :similar_client_last_name
      attribute :similar_client_bday
      attribute :similar_client_gender
      attribute :similar_client_gender_name
      attribute :similar_client_institution_id
      attribute :similar_client_institution_name
      attribute :similar_client_institution_school_no
      attribute :similar_client_institution_email
      attribute :similar_client_institution_fax
      attribute :similar_client_institution_website
      attribute :similar_client_institution_phone
      attribute :similar_client_institution_address_street
      attribute :similar_client_institution_address_city
      attribute :similar_client_institution_address_region
      attribute :similar_client_institution_address_zip
      attribute :similar_client_institution_address_country
      attribute :similar_client_institution_address_number


      filter :client_first_name
      filter :client_last_name
      filter :client_bday
      filter :client_gender

    end
  end
end