class ChronologicalOrderValidator < ActiveModel::EachValidator
  def validate_each(record, attribute, value)
    fields = options[:fields] || (raise StandardError.new('Missing option: fields'))
    last_field = nil
    last_value = nil
    fields.each do |field|
      if field.is_a?(String) && field.include?('.')
        parts = field.split('.')
        last_part = parts.pop
        receiver = record
        parts.each do |part|
          receiver = receiver.public_send(part)
        end
        value = receiver.public_send("#{last_part}_before_type_cast")
      else
        value = record.public_send("#{field}_before_type_cast")
      end
      next unless value.present?
      unless value.is_a? Date
        value = Date.parse(value)
      end
      if last_value.present? && last_value > value
        record.errors[attribute] << (options[:message] || "Bitte in chronologischer Reihenfolge bleiben")
        return
      end
      last_field = field
      last_value = value
    end
  end
end
