require File.expand_path('../boot', __FILE__)

require 'rails/all'
#require 'pdfkit'

# Require the gems listed in Gemfile, including any gems
# you've limited to :test, :development, or :production.
Bundler.require(*Rails.groups)

# this causes actionmailer not to use the queue 'mailers' but the queue 'default'
# until problems occur, we only use a single queue for background processes
class ActionMailer::DeliveryJob
  queue_as :default
end

module Vschoolboard
  class Application < Rails::Application
    # do not wrap form errors in a <div class="fields_with_errors">
    config.action_view.field_error_proc = Proc.new { |html_tag, instance|
      html_tag
    }

    #config.middleware.use PDFKit::Middleware
    # Settings in config/environments/* take precedence over those specified here.
    # Application configuration should go into files in config/initializers
    # -- all .rb files in that directory are automatically loaded.
    config.x.local_env = config_for(:local_env)
    config.before_configuration do
      config.x.local_env.each do |key,value|
        ENV[key.to_s] = value.to_s
      end
    end

    # Set Time.zone default to the specified zone and make Active Record auto-convert to this zone.
    # Run "rake -D time" for a list of tasks for finding time zone names. Default is UTC.
    config.time_zone = 'Berlin'

    # The default locale is :en and all translations from config/locales/*.rb,yml are auto loaded.
    # config.i18n.load_path += Dir[Rails.root.join('my', 'locales', '*.{rb,yml}').to_s]
    config.i18n.default_locale = :de

    # Do not swallow errors in after_commit/after_rollback callbacks.
    config.active_record.raise_in_transactional_callbacks = true

    # As the application is tied to postgresql right now and the usual schema.rb does not support several db features
    # we use a raw sql dump
    config.active_record.schema_format = :sql

    # custom validators can be placed in /app/validators/ and will be accessible in models
    config.autoload_paths += %W["#{config.root}/app/validators/"]

    # serializers can be placed in /app/serializers/
    config.autoload_paths += %W["#{config.root}/app/serializers/"]
    config.autoload_paths += %W["#{config.root}/app/resources/"]
    config.autoload_paths += %W["#{config.root}/app/grids/"]
    config.autoload_paths += %W["#{config.root}/app/authorization/"]

    config.assets.precompile += Ckeditor.assets
    config.assets.precompile += %w( ckeditor/* )
    config.autoload_paths += %W(#{config.root}/app/models/ckeditor)

    config.active_job.queue_adapter = :queue_classic

  end
end

