class AddDiagnosticMeasurements < ActiveRecord::Migration
  def up
    drop_table :portfolio_item_class_diagnostic_documents
    # they are like measurements, but with results
    # entities that can have results are: class_diagnostic, diagnostic and diagnostic_measurement
    create_table :portfolio_item_diagnostic_measurements do |t|
      t.date :begin
      t.text :content
      t.references :portfolio_item, index: {name: 'idx_pidm_on_pi_id'}
      t.boolean :results_closed
      t.timestamps
    end
    # this table contains informal results, but also has many portfolio_item_diagnostic_results (the formal ones)
    create_table :portfolio_item_diagnostic_informal_results do |t|
      t.text :result
      # this can be any entity with results: class_diagnostic, diagnostic and diagnostic_measurement
      t.references :diagnostic_result_acceptor, polymorphic: true, index: {name: 'idx_pidir_on_dia_id'}
      t.boolean :subject_involved
      t.timestamps
    end
    # these are the formal results, they belong to an informal result now
    add_reference :portfolio_item_diagnostic_results, :portfolio_item_diagnostic_informal_result, index: {name: 'idx_pidr_on_pidir_id'}
    # for each portfolio item diagnostic, there must be an informal result created and the formal results must be attached to this informal result
    say_with_time 'copy existing diagnostic results' do
      Portfolio::ItemDiagnostic.all.each do |id|
        ir = id.informal_results.build
        ir.results = Portfolio::ItemDiagnosticResult.where(portfolio_item_diagnostic_id: id.id )
        ir.save!
        say("Moved portfolio_item_diagnostic_result.id (#{ir.results.ids.join(', ')}) to new portfolio_item_diagnostic_informal_result.id (#{ir.id})" ,true)
      end
    end

    remove_reference :portfolio_item_diagnostic_results, :portfolio_item_diagnostic
    # just a list of documents for an informal result
    create_table :portfolio_item_diagnostic_informal_result_documents do |t|
      t.string :file_name
      t.references :portfolio_item_diagnostic_informal_result, index: {name: 'idx_pidird_on_pidir_id'}
      t.timestamps
    end
    # these are the involved people that can be attached to a result
    create_table :portfolio_item_diagnostic_informal_result_teachers do |t|
      t.references :teacher, index: {name: 'idx_pidirt_on_t_id'}
      t.references :portfolio_item_diagnostic_informal_result, index: {name: 'idx_pidirt_on_pidir_id'}
    end
    create_table :portfolio_item_diagnostic_informal_result_guardians do |t|
      t.references :guardian, index: {name: 'idx_pidirg_on_g_id'}
      t.references :portfolio_item_diagnostic_informal_result, index: {name: 'idx_pidirg_on_pidir_id'}
    end
    create_table :portfolio_item_diagnostic_informal_result_supporters do |t|
      t.references :supporter, index: {name: 'idx_pidirs_on_s_id'}
      t.references :portfolio_item_diagnostic_informal_result, index: {name: 'idx_pidirs_on_pidir_id'}
    end
    create_table :portfolio_item_diagnostic_informal_result_people do |t|
      t.string :name
      t.references :portfolio_item_diagnostic_informal_result, index: {name: 'idx_pidirp_on_pidir_id'}
    end
  end

  def down
    drop_table :portfolio_item_diagnostic_informal_result_people
    drop_table :portfolio_item_diagnostic_informal_result_supporters
    drop_table :portfolio_item_diagnostic_informal_result_guardians
    drop_table :portfolio_item_diagnostic_informal_result_teachers
    drop_table :portfolio_item_diagnostic_informal_result_documents
    say('dropped table with referenced files. Following directory contents are orphaned and now unknown to the schoolboard: public/uploads/portfolio_diagnostic_informal_results')
    add_reference :portfolio_item_diagnostic_results, :portfolio_item_diagnostic, index: {name: 'idx_pidr_on_pid_id'}
    say_with_time 'copy diagnostic results. This may lead to duplicated entries for certain formal results.' do
      Portfolio::ItemDiagnosticResult.all.each do |result|
        diagnostic = result.portfolio_diagnostic_informal_result.diagnostic_result_acceptor
        if diagnostic.is_a? Portfolio::ItemDiagnostic
          result.portfolio_item_diagnostic_id=diagnostic.id
          result.save!
          say("Moved portfolio_item_diagnostic_result.id (#{result.id}) to existing portfolio_item_diagnostic.id (#{diagnostic.id})" ,true)
        end
      end
    end
    remove_reference :portfolio_item_diagnostic_results, :portfolio_item_diagnostic_informal_result
    drop_table :portfolio_item_diagnostic_informal_results
    drop_table :portfolio_item_diagnostic_measurements
    create_table :portfolio_item_class_diagnostic_documents do |t|
      t.string :file_name
      t.references :portfolio_item_class_diagnostic
      t.timestamps
    end
  end
end
