class RoleBasedSchoolAdmins < ActiveRecord::Migration
  def up
    add_column :teacher_institution_relations, :is_school_admin, :boolean, default: false, null: false
    execute <<-SQL
      update teacher_institution_relations set is_school_admin=true
      from teacher_institution_categories
      where teacher_institution_categories.id = teacher_institution_category_id
      and teacher_institution_categories.is_school_admin=true;
    SQL
    types = %w(
      AuthItemCreateClientLeadedInstitutionsAsTeacher
      AuthItemDestroyClientLeadedInstitutionsAsTeacher
      AuthItemCreateClientGroupLeadedInstitutionsAsTeacher
      AuthItemDestroyClientGroupLeadedInstitutionsAsTeacher
      AuthItemUpdateInstitutionSchoolLeadedInstitutionsAsTeacher
      AuthItemCreateTeacherLeadedInstitutionsAsTeacher
      AuthItemUpdateTeacherLeadedInstitutionsAsTeacher
      AuthItemUpdateRolesTeacherLeadedInstitutionsAsTeacher
      AuthItemDestroyTeacherLeadedInstitutionsAsTeacher
      AuthItemReadNotificationLeadedInstitutionsAsTeacher
      AuthItemDestroyNotificationLeadedInstitutionsAsTeacher
    )
    role = AuthRole.new({
       name: 'Schuladmin',
       description: 'Schuladministratoren haben an den Schulen, an denen Sie als Schuladmin eingetragen sind, zusätzliche Rechte. Beachten: Ein Benutzer braucht diese Rolle, muss aber auch als Schuladmin an einer Schule arbeiten.',
       user_type: 'Teacher',
       is_default_role: false,
    })
    # AuthItem<Operation><model><scope>As<user_type>
    role.auth_items = AuthItem.where(type: types).all
    role.save!
    Teacher.distinct.joins(:teacher_institution_relations).where('teacher_institution_relations.is_school_admin=true').each do |t|
      if t.identity
        t.identity.auth_roles << role
        t.identity.save!
      end
    end
    teacher_role = AuthRole.where(user_type: 'Teacher', is_default_role: true).take!
    teacher_role.auth_items.delete(AuthItem.where(type: types).all)
    remove_column :teacher_institution_categories, :is_school_admin
  end

  def down
    types = %w(
      AuthItemCreateClientLeadedInstitutionsAsTeacher
      AuthItemDestroyClientLeadedInstitutionsAsTeacher
      AuthItemCreateClientGroupLeadedInstitutionsAsTeacher
      AuthItemDestroyClientGroupLeadedInstitutionsAsTeacher
      AuthItemUpdateInstitutionSchoolLeadedInstitutionsAsTeacher
      AuthItemCreateTeacherLeadedInstitutionsAsTeacher
      AuthItemUpdateTeacherLeadedInstitutionsAsTeacher
      AuthItemUpdateRolesTeacherLeadedInstitutionsAsTeacher
      AuthItemDestroyTeacherLeadedInstitutionsAsTeacher
      AuthItemReadNotificationLeadedInstitutionsAsTeacher
      AuthItemDestroyNotificationLeadedInstitutionsAsTeacher
    )
    add_column :teacher_institution_categories, :is_school_admin, :boolean
    teacher_role = AuthRole.where(user_type: 'Teacher', is_default_role: true).take!
    AuthItem.where(type: types).each do |item|
      teacher_role.auth_items << item
    end
    teacher_role.save!
    role = AuthRole.where({name: 'Schuladmin'}).take!
    Teacher.distinct.joins(:teacher_institution_relations).where('teacher_institution_relations.is_school_admin=true').each do |t|
      t.identity.auth_roles.delete(role)
    end
    role.destroy
    execute <<-SQL
      update teacher_institution_categories set is_school_admin=true
      where name ilike '%admin%';
    SQL
    remove_column :teacher_institution_relations, :is_school_admin
  end
end
