class AddFoldersToInstitutionDocuments < ActiveRecord::Migration
  def down
    execute 'DROP TRIGGER IF EXISTS institution_documents_maintain_tree_consistency ON institution_documents;'
    execute 'DROP FUNCTION IF EXISTS institution_document_hierarchies_update()'
    remove_column :institution_documents, :parent_id, :integer
    drop_table :institution_document_hierarchies
  end
  def up
    create_table :institution_document_hierarchies, id: false do |t|
      t.integer :parent_id, index: true
      t.integer :child_id, index: true
      t.integer :level, index: true
    end
    add_column :institution_documents, :parent_id, :integer
    change_column :institution_documents, :file, :string , null: true
    execute 'WITH RECURSIVE institution_document_hierarchies_cte(child_id,parent_id,level) AS (
      SELECT institution_documents.id, institution_documents.parent_id, 1
      FROM institution_documents WHERE institution_documents.parent_id IS NOT NULL
      UNION
      SELECT institution_document_hierarchies_cte.child_id, institution_documents.parent_id, institution_document_hierarchies_cte.level + 1
      FROM institution_document_hierarchies_cte JOIN institution_documents  ON institution_document_hierarchies_cte.parent_id = institution_documents.id
      WHERE institution_documents.parent_id IS NOT NULL
    )  INSERT INTO institution_document_hierarchies SELECT institution_document_hierarchies_cte.parent_id,institution_document_hierarchies_cte.child_id,institution_document_hierarchies_cte.level FROM institution_document_hierarchies_cte;'
    execute "
      CREATE OR REPLACE FUNCTION institution_document_hierarchies_update() RETURNS TRIGGER
      AS $institution_document_hierarchies_update$
          BEGIN
              IF (TG_OP = 'DELETE') THEN
                DELETE FROM institution_document_hierarchies WHERE child_id=OLD.id OR parent_id=OLD.id;
                UPDATE institution_documents SET parent_id=OLD.parent_id WHERE parent_id=OLD.id;
              ELSIF (TG_OP = 'UPDATE') THEN
                -- Rebuild the complete hierarchy, consider rebuilding partly when performance degrades
                DELETE FROM institution_document_hierarchies;
                WITH RECURSIVE institution_document_hierarchies_cte(child_id,parent_id,level) AS (
                  SELECT institution_documents.id, institution_documents.parent_id, 1
                  FROM institution_documents WHERE institution_documents.parent_id IS NOT NULL
                  UNION
                  SELECT institution_document_hierarchies_cte.child_id, institution_documents.parent_id, institution_document_hierarchies_cte.level + 1
                  FROM institution_document_hierarchies_cte JOIN institution_documents  ON institution_document_hierarchies_cte.parent_id = institution_documents.id
                  WHERE institution_documents.parent_id IS NOT NULL
                )  INSERT INTO institution_document_hierarchies SELECT institution_document_hierarchies_cte.parent_id,institution_document_hierarchies_cte.child_id,institution_document_hierarchies_cte.level FROM institution_document_hierarchies_cte;
              ELSIF (TG_OP = 'INSERT') THEN
                -- Rebuild the complete hierarchy, consider rebuilding partly when performance degrades
                DELETE FROM institution_document_hierarchies;
                WITH RECURSIVE institution_document_hierarchies_cte(child_id,parent_id,level) AS (
                  SELECT institution_documents.id, institution_documents.parent_id, 1
                  FROM institution_documents WHERE institution_documents.parent_id IS NOT NULL
                  UNION
                  SELECT institution_document_hierarchies_cte.child_id, institution_documents.parent_id, institution_document_hierarchies_cte.level + 1
                  FROM institution_document_hierarchies_cte JOIN institution_documents  ON institution_document_hierarchies_cte.parent_id = institution_documents.id
                  WHERE institution_documents.parent_id IS NOT NULL
                )  INSERT INTO institution_document_hierarchies SELECT institution_document_hierarchies_cte.parent_id,institution_document_hierarchies_cte.child_id,institution_document_hierarchies_cte.level FROM institution_document_hierarchies_cte;
              END IF;
              RETURN NULL;
          END;
      $institution_document_hierarchies_update$ LANGUAGE plpgsql;
      CREATE TRIGGER institution_documents_maintain_tree_consistency
      AFTER INSERT OR UPDATE OR DELETE ON institution_documents
          FOR EACH ROW EXECUTE PROCEDURE institution_document_hierarchies_update();"

  end
end
