#!/bin/bash
#-----------------------------------------------------------------------------------------------------------------------
#
# This is the start script for the message queue
# The worker will start in foreground, it is adviseable to put this script into init / systemd
# so, a respawn / restart directive can be applied
#
# USAGE: mqctl.sh { start | stop | restart }  { production | development }
#
#-----------------------------------------------------------------------------------------------------------------------

INSTANZ="Vschoolboard Message Queue"
SCRIPT=$(readlink -f "$0")
SCRIPTPATH=$(dirname "$SCRIPT")
export RAILS_ENV=$2
cd $SCRIPTPATH
Pidfile=tmp/pids/mq.pid

if [ -f $Pidfile ]
then
	Pid=`cat $Pidfile`
fi

case "$1" in
'start')
		if [ -f $Pidfile ] ; then
				if test `ps -e | grep -c $Pid` = 1; then
						echo "Not starting $INSTANZ - instance already running with PID: $Pid"
				else
						echo "Starting $INSTANZ"
						bundle exec rake qc:work &
						echo "$!" > "$Pidfile"
						wait ${!}
				fi
		else
					echo "Starting $INSTANZ"
          bundle exec rake qc:work &
          echo "$!" > "$Pidfile"
          wait ${!}
		fi
		;;

'stop')
		if [ -f $Pidfile ] ; then
				echo "stopping $INSTANZ"
				kill `cat $Pidfile`
				rm "$Pidfile"
		else
				echo "Cannot stop $INSTANZ - no Pidfile found!"
		fi
		;;

'restart')
    echo "restarting $INSTANZ"
    if test `ps -e | grep -c $Pid` = 1; then
				kill `cat $Pidfile`
				bundle exec  rake qc:work &
        echo "$!" > "$Pidfile"
        wait ${!}
		else
				bundle exec rake qc:work &
        echo "$!" > "$Pidfile"
        wait ${!}
		fi
		;;

*)
		echo "usage: $0 { start | stop | restart } { production | development }"
		;;

esac
exit 0